<?php
include('head.php');
?>
  <div style="text-align:center; font-size:35px;">
    O Programu / Teorijska podloga
  </div>
  <br />
  <br />
  Tok izvršavanja programa možemo podeliti na sledeće logičke celine:
  <ul>
    <li>Generisanje vrednosti</li>	
    <li>Unos podataka</li>
    <li>Obrada i prikaz rezultata</li>
  </ul>
  <br />
  <h2>Generisanje vrednosti</h2>
  Ono na šta posebno treba obratiti pažnju je samo generisanje podataka, tačnije sčučajnih brojeva.<br />
  Obzirom da su kompjuteri deterministički uređaji, svaki pokušaj kreiranja generator slučajnih brojeva je teorijski nemoguć.<br />
  Svaka situacija gde nam se traži slučajni broj predstavlja problem.<br />
  Problem potrebe slučajnih brojeva rešavamo uvođenjem funkcija generatora pseudo-slučajnih brojeva, koji generišu nizove ovakvh brojeva.<br />
  Iako se zna da se ovi brojevi posle izvesnog vremena ponavljaju, ovo nam ne predstavlja prevelik problem, obzirom da su periode njihovih ponavljanja poprilično velike, kod novijih generatora.<br />
  Na primer, generator u jeziku PHP, u kome je najvećim delom i napisan ovaj program, perioda ponavljanja je 2<sup>219937</sup>.<br />
  Algoritam koji stoji iza ovog generatora se zove <i>Mersenov tvister</i>.<br />
  <br />
  Kako bi izbegli slučajno ponavljanje pitanja u kvizu, koje bi remetilo realnost ocene, brojeve ne generišemo nasumično.<br />
  Brojevi su generisani kao niz od 1 do 20, koliko pitanja i imamo, a zatim ih "mešamo".<br />
  Mešanje niza vrši funkcija <i>shuffle()</i> koja se oslanja na <i>Mersenov tvister</i>.<br />
  Obzirom da u nizu imamo 20 pitanja, iz njega "odsecamo" poslednjih 10, kako bi smo ga sveli na 10 traženih nasumično odabranih pitanja.<br />
  Nasumičnost mogućih odgovora za svako pitanje se postiže mešanjem odgovora u samom SQL upitu ključnim rečima <code>ODRED BY RANDOM()</code>.<br />
  Iza funkcije <i>RANDOM()</i>, koja je ugrađena u MYSQL server na koji se program oslanja, takođe stoji <i>Mersenov tvister</i>.<br />
  Nakon što je niz brojeva generisan, pretvaramo ga u String, i kao takav zapisujemo u COOKIE na računaru korisnika.
  <br /><br />
  <h3>Mersenov tvister</h3>
  Na osnovu metoda koji koriste, softveri za generisanje pseudoslučajnih brojeva se mogu podeliti na:
  <ul>
    <li>Linearni kongruentni generator- linearni generator slučajnih brojeva</li>	
    <li>Fibonačijevog generator- dobili su ime zbog sličnosti formule koju koriste sa formulom koja se koristi za dobijanje Fibonačijevog niza</li>
    <li>Generator linearno pomerajućeg registra - brojevi se ne dobijaju aritmetičkim već logičkim operacijama</li>
  </ul>
  Mersenov tvister je relativno nov algoritam za generisanje pseudoslučajnih brojeva, razvijen od strane Makota Matsumotoa i Takuji Nišimura 1997. koji se koristi kod većine modernih programskih jezika kao ugrađena rand() funkcija.<br />
  On spada pod generatore linearno pomerajućeg registra Frobeniusove (racionalne) normalne forme. Karakterizuje ga:
  <ul>
    <li>w – dužina reči</li>
    <li>n – nivo rekurzije</li>
    <li>m – srednja reč 1≤m≤n</li>
    <li>r – mesto razdvajanja reči – broj bitova u donjoj bitmasci 0≤r≤w-1</li>
    <li>a – koeficijent Frobeniusove (racionalne) normalne forme matrice</li>
    <li>b,c – bitmaske za algoritme ovog tipa</li>
    <li>s,t – pomaci bitova</li>
    <li>u,l – dodatni Mersen tvister pomaci bitova</li>
  </ul>
  Pri čemu važi da je 2<sup>nw-r</sup>-1 Mersenov prost broj.<br />
  Ono što je najvažnije kod ovog algoritma da za reč dužine k generiše brojeve sa skoro uniformnom raspodelom u opsegu [0,(2<sup>k</sup>− 1)].<br />
  Implementacija algoritma:
  <code>
  <br />
  // Create a length 624 array to store the state of the generator<br />
  int[0..623] MT<br />
  int index = 0<br />
  // Initialize the generator from a seed<br />

  function initializeGenerator(int seed) {<br />
    MT[0] := seed<br />
    for i from 1 to 623 { // loop over each other element<br />
      MT[i] := last 32 bits of(1812433253 * (MT[i-1] xor (right shift by 30<br />
      bits(MT[i-1]))) + i) // 0x6c078965<br />
    }<br />
  }<br />
  // Extract a tempered pseudorandom number based on the index-th value,<br />
  // calling generateNumbers() every 624 numbers<br />

  function extractNumber() {<br />
    if index == 0 {<br />
      generateNumbers()<br />
    }<br />
    int y := MT[index]<br />
    y := y xor (right shift by 11 bits(y))<br />
    y := y xor (left shift by 7 bits(y) and (2636928640)) // 0x9d2c5680<br />
    y := y xor (left shift by 15 bits(y) and (4022730752)) // 0xefc60000<br />
    y := y xor (right shift by 18 bits(y))<br />

    index := (index + 1) mod 624<br />
    return y<br />
  }<br />

  // Generate an array of 624 untempered numbers<br />
  function generateNumbers() {<br />
    for i from 0 to 623 {<br />
      int y := 32nd bit of(MT[i]) + last 31 bits of(MT[(i+1) mod 624])<br />
      MT[i] := MT[(i + 397) mod 624] xor (right shift by 1 bit(y))<br />
      if (y mod 2) == 1 { // y is odd<br />
	MT[i] := MT[i] xor (2567483615) // 0x9908b0df<br />
      }<br />
    }<br />
  }<br />
  </code>
  <br /><br /><br />
  <h2>Unos podataka</h2>
  Sam unos podataka se vrši jednostavnim klikom na ispravan odgovor.<br />
  Pitanja se korisniku postavljaju onim redom kojim su zapisana u kolaču na korisnikovom kompjuteru.<br />
  Klikom na tačan odgovor se pokreće JavaScript koji zapisuje korisnikove odgovore u novi kolač, koji će kasnije php obraditi prilikom računanja ocene.<br />
  Nakon toga se to pitanje izbacuje iz niza postavljenih pitanja, kako ga ne bi smo ponovo postavili.<br />
  <br /><br /><br />
  <h2>Obrada i prikaz podataka</h2>
  Nakon što je korisnik odgovorio na sva pitanja koja su mu postavljena, poziva se PHP skript koji iz kolača čita korisnikove odgovore, i poredi ih sa odgovorima koji su u bazi naznačeni kao tačni.<br />
  Skripta korisniku ispisuje da li je odgovor tačan ili ne, i nudi mogućnost prikaza tačnog odgovora sa detaljnim objašnjenjem.<br />
  Nakon obrada svih postavljenih pitanja, i dobijenih odgovora, korisniku se prikazuje i ocena koju je zaslužio, kao i mogućnost da pokrene novi test.<br />
  <br /><br /><br />
<?php
include('foot.php');
?>